/*
 * (C) Copyright 2008-2009
 * Marcel Ziswiler, Noser Engineering, marcel.ziswiler@noser.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>		/* do not change order of include file */
#include <asm/arch/pxa-regs.h>
#include <command.h>		/* needs to be after common.h */
#include <environment.h>
#include <net.h>		/* for eth_initialize_mac() */

DECLARE_GLOBAL_DATA_PTR;

#define		RH_A_PSM	(1 << 8)	/* power switching mode */
#define		RH_A_NPS	(1 << 9)	/* no power switching */

typedef struct {
	u32 l : 5;
	u32 n : 4;
	u32 t : 1;
	u32 b : 1;
	u32 a : 1;
	u32 k1db2 : 1;
	u32 k0db2 : 1;
	u32 k0db4 : 1;
	u32 ht : 1;
	u32 res : 16; /* Reserved for future usage */
} speed_t;

extern struct serial_device serial_ffuart_device;
extern struct serial_device serial_btuart_device;
extern struct serial_device serial_stuart_device;

/* ------------------------------------------------------------------------- */

/*
 * Miscelaneous platform dependent initialisations
 */

/* Save the environment only (not the other data in same sector).
 * Essentially taken from saveenv in env_flash.c
 * Leave unprotected.
 */
static int save_env_only(void)
{
	extern env_t *env_ptr;
	int len, rc;
	ulong end_addr;
	const ulong flash_sect_addr = CONFIG_ENV_ADDR - CONFIG_FIX_PARAM_SIZE;
	uchar *env_buffer = (uchar *)env_ptr;
	int rcode = 0;

	len = CONFIG_ENV_SIZE;
	end_addr = flash_sect_addr + CONFIG_ENV_SECT_SIZE - 1;

	if (flash_sect_protect(0, flash_sect_addr, end_addr))
		return 1;

	puts("Erasing Flash...");
	if (flash_sect_erase(flash_sect_addr, end_addr))
		return 1;

	puts("Writing to Flash... ");
	rc = flash_write((char *)env_buffer, flash_sect_addr + CONFIG_FIX_PARAM_SIZE, len);
	if (rc != 0) {
		flash_perror(rc);
		rcode = 1;
	} else
		puts("done\n");

	return rcode;
}

int do_resetcfg(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char *p, *ep;
	unsigned long nr;
	speed_t speed = { l: 16, n: 3, t: 1, b: 1, a: 1,
			  k1db2: 1, k0db2: 0, k0db4: 0, ht: 0, res: 0 };
	unsigned char mac_addr[6] = {0, 0x14, 0x2d, 0, 0, 0};
	union {
		u32 speed_val;
		speed_t speed_param;
	} speed_u;
	struct {
		uchar eth_addr[6];
		uchar res[2];
		u32 freq_cfg;
	} cfg_block = { { 0, 0, 0, 0, 0, 0 }, { 0, 0 }, 0 };

	if (argc != 2) {
		printf("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	p = strchr(argv[1], '-');
	if (p == NULL) {
		printf("Wrong serial-string format: Usage:\n%s\n",
		       cmdtp->usage);
		return 1;
	}
	*p = '\0';  /* terminate first number */
	++p;        /* p points now to second number */

	nr = simple_strtoul(argv[1], &ep, 10);
	if (ep == argv[1] || *ep != '\0') {
		printf("Wrong serial-string format: Usage:\n%s\n",
		       cmdtp->usage);
		return 1;
	}
	if (nr == 1) {
		speed.n = 3;
		puts("Setting speed to 312MHz\n");
	} else if (nr == 2) {
		speed.n = 5;
		puts("Setting speed to 520MHz\n");
	} else {
		printf("Illegal first part of serial-string: %s\n", argv[1]);
		return 1;
	}
	speed_u.speed_param = speed;

	nr = simple_strtoul(p, &ep, 10);
	if (ep == p || *ep != '\0') {
		printf("Wrong serial-string format: Usage:\n%s\n",
		       cmdtp->usage);
		return 1;
	}

	mac_addr[5] = nr & 0xff;
	mac_addr[4] = (nr >> 8) & 0xff;
	mac_addr[3] = (nr >> 16) & 0xff;

	memcpy(cfg_block.eth_addr, mac_addr, 6);
	cfg_block.freq_cfg = speed_u.speed_val;

#if 0
	puts("Config block to write: ");
	for (p = (char *)&cfg_block; p != (char *)(&cfg_block + 1); ++p)
		printf("%02x ", *p);
	puts("\n");
#else
	if (save_env_only() != 0) {
		puts("Writing environment sector failed.\n");
		return 1;
	}
	if (flash_write((char *)&cfg_block,
			CONFIG_ENV_ADDR + CONFIG_ENV_SIZE,
			sizeof(cfg_block)) != 0) {
		puts("Writing config block failed.\n");
		return 1;
	}
#endif

	/* try to re-protect */
	(void) flash_sect_protect(1, CONFIG_ENV_ADDR - CONFIG_FIX_PARAM_SIZE,
				  CONFIG_ENV_ADDR + CONFIG_ENV_SECT_SIZE -
				  CONFIG_FIX_PARAM_SIZE - 1);

	return 0;
}

#ifdef CFG_DEEP_DEBUG
static void dump_registers(void)
{
	printf("CCCR = 0x%08x\n", CCCR);
	printf("MSC0 = 0x%08x\n", MSC0);
	printf("MSC1 = 0x%08x\n", MSC1);
	printf("MSC2 = 0x%08x\n", MSC2);

	printf("GPDR0 = 0x%08x\n", GPDR0);
	printf("GPDR1 = 0x%08x\n", GPDR1);
	printf("GPDR2 = 0x%08x\n", GPDR2);
	printf("GPDR3 = 0x%08x\n", GPDR3);

	printf("GPLR0 = 0x%08x\n", GPLR0);
	printf("GPLR1 = 0x%08x\n", GPLR1);
	printf("GPLR2 = 0x%08x\n", GPLR2);
	printf("GPLR3 = 0x%08x\n", GPLR3);

	printf("GAFR0_L = 0x%08x\n", GAFR0_L);
	printf("GAFR0_U = 0x%08x\n", GAFR0_U);
	printf("GAFR1_L = 0x%08x\n", GAFR1_L);
	printf("GAFR1_U = 0x%08x\n", GAFR1_U);
	printf("GAFR2_L = 0x%08x\n", GAFR2_L);
	printf("GAFR2_U = 0x%08x\n", GAFR2_U);
	printf("GAFR3_L = 0x%08x\n", GAFR3_L);
	printf("GAFR3_U = 0x%08x\n", GAFR3_U);
}
#endif /* CFG_DEEP_DEBUG */

int board_init(void)
{
	/* memory and cpu-speed are setup before relocation */
	/* so we do _nothing_ here */

	/* arch number of colibri */
	gd->bd->bi_arch_number = MACH_TYPE_COLIBRI;

	/* adress of boot parameters */
	gd->bd->bi_boot_params = 0xa0000100;

	return 0;
}

int board_late_init(void)
{
	unsigned char mac_addr[6] = {0, 0, 0, 0, 0, 0};
	unsigned char mac_addrff[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
	char env_str[64];
	u32 cfg_addr = CONFIG_ENV_ADDR + CONFIG_ENV_SECT_SIZE -
		       CONFIG_FIX_PARAM_SIZE - CONFIG_FIX_PARAM_SIZE;
	union {
		u32 speed_val;
		speed_t speed_param;
	} speed_u;
	int speed_offset = 8;
	int env_changed = 0;
	char *speed_str;
	char config_available = 0;

#if defined(CONFIG_SERIAL_MULTI)
	char *console=getenv("boot_console");

	if ((strcmp(console,"serial_btuart") == 0) ||
		(strcmp(console,"serial_stuart") == 0) ||
		(strcmp(console,"serial_ffuart") == 0)) {
			setenv("stdout",console);
			setenv("stdin", console);
			setenv("stderr",console);
	} else {
		setenv("stdout", "serial");
		setenv("stdin", "serial");
		setenv("stderr", "serial");
	}
#endif

	/* handle production parameter config block */
	if (memcmp(mac_addrff, (unsigned char *)cfg_addr, 6) == 0
	    || memcmp(mac_addr, (unsigned char *)cfg_addr, 6) == 0) {
		cfg_addr = CONFIG_ENV_ADDR - CONFIG_FIX_PARAM_SIZE + 8;
		if (memcmp(mac_addrff, (unsigned char *)cfg_addr, 6) == 0
		    || memcmp(mac_addr, (unsigned char *)cfg_addr, 6) == 0) {
			config_available = 0;
			printf("Missing Colibri config block\n");
		} else {
			config_available = 1;
			speed_offset = 12;
		}
		
	} else {
		config_available = 1;
	}
	if (config_available == 1) {
		/* set Ethernet MAC address from config block if not already
		   set */
		if (memcmp(mac_addr, gd->bd->bi_enetaddr, 6) == 0) {
			memcpy(mac_addr, (unsigned char *)cfg_addr, 6);
			sprintf(env_str, "%02x:%02x:%02x:%02x:%02x:%02x",
				mac_addr[0], mac_addr[1], mac_addr[2],
				mac_addr[3], mac_addr[4], mac_addr[5]);
			setenv("ethaddr", env_str);
			env_changed = 1;
		}

		/* set CPU speed from config block if not already set */
		speed_str = getenv("pxaspeed");
		if (speed_str == NULL) {
			speed_u.speed_param = *((speed_t *)(cfg_addr+speed_offset));
			if ((speed_u.speed_param.l < 7) ||
			    (speed_u.speed_param.l > 16)) {
				printf("illegal CPU speed parameter; "
				       "using safe default\n");
				speed_u.speed_val = 0x0670;
			}
			sprintf(env_str, "0x%08x", speed_u.speed_param);
			setenv("pxaspeed", env_str);
			env_changed = 1;
		} else {
			speed_u.speed_val = simple_strtol(speed_str, NULL, 0);
		}
		pxa27x_set_core_freq(speed_u.speed_param.l,
				     speed_u.speed_param.n,
				     speed_u.speed_param.t,
				     speed_u.speed_param.b,
				     1); /* for specific steppings only! */
				     /* speed_u.speed_param.a); */

		if (env_changed)
			saveenv();

		printf("CPU speed: %dkHz\n", pxa27x_get_freq());
	}

	/* un-protect U-Boot sector (because Linux doesn't do it correctly) */
	(void)flash_sect_protect(0, CFG_FLASH_BASE, PHYS_FLASH_SECT_SIZE-1);

#if defined(CONFIG_FASTBOOT_GPIO)
	GPCR(CONFIG_FASTBOOT_GPIO) |= GPIO_bit(CONFIG_FASTBOOT_GPIO);
	GPDR(CONFIG_FASTBOOT_GPIO) &= ~GPIO_bit(CONFIG_FASTBOOT_GPIO);
	if (GPLR(CONFIG_FASTBOOT_GPIO) & GPIO_bit(CONFIG_FASTBOOT_GPIO))
		gd->flags |= GD_FLG_SILENT | GD_FLG_FASTBOOT;
	else
		gd->flags &= ~(GD_FLG_SILENT | GD_FLG_FASTBOOT);
#endif /* CONFIG_FASTBOOT_GPIO */

#if defined(CONFIG_ALTBOOT_GPIO)
	GPCR(CONFIG_ALTBOOT_GPIO) |= GPIO_bit(CONFIG_ALTBOOT_GPIO);
	GPDR(CONFIG_ALTBOOT_GPIO) &= ~GPIO_bit(CONFIG_ALTBOOT_GPIO);
	if (GPLR(CONFIG_ALTBOOT_GPIO) & GPIO_bit(CONFIG_ALTBOOT_GPIO))
		gd->flags |= GD_FLG_ALTBOOT;
	else
		gd->flags &= ~(GD_FLG_ALTBOOT);
#endif /* CONFIG_ALTBOOT_GPIO */

#ifdef CFG_DEEP_DEBUG
	dump_registers();
#endif /* CFG_DEEP_DEBUG */

#if defined(CONFIG_DRIVER_DM9000)
	/*
	 * initialize Ethernet MAC address as Linux does so-called mac-from-dev
	 */
	eth_initialize_mac(gd->bd);
#endif /* CONFIG_CMD_NET */

	return 0;
}

int dram_init(void)
{
	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size = PHYS_SDRAM_1_SIZE;

	return 0;
}

void get_board_serial(struct tag_serialnr *serialnr)
{
	union {
		struct tag_serialnr tag;
		unsigned char enetaddr[6];
	} join;

	memset(&join, 0, sizeof(join));
	memcpy(join.enetaddr, gd->bd->bi_enetaddr, 6);
	*serialnr = join.tag;
}

struct serial_device *default_serial_console (void)
{
	return &serial_ffuart_device;
}

U_BOOT_CMD(
	reset_config,    2,    0,    do_resetcfg,
	"reset_config  - rewrites the production configuration block on a"
	" Colibri\n",
	"xxx-yyyyyyy"
	"    - serial number from the label on the Colibri module"
);

void usb_board_init(void)
{
	/* Set the Power Control Polarity Low and Power Sense
	   Polarity Low to active low. */
	UHCHR = (UHCHR | UHCHR_PCPL | UHCHR_PSPL | UHCHR_SSEP3) &
		~(UHCHR_SSEP1 | UHCHR_SSEP2 | UHCHR_SSE);

	UP2OCR = UP2OCR_HXOE | UP2OCR_HXS;	/* use port 2 as host */
	UP2OCR |= UP2OCR_DPPDE | UP2OCR_DMPDE;	/* enable D +/- pull-down */

	UHCHR |= UHCHR_FSBIR;

	while (UHCHR & UHCHR_FSBIR);

	UHCHIE = (UHCHIE_UPRIE | UHCHIE_RWIE);

	/* Clear any OTG Pin Hold */
	if (PSSR & PSSR_OTGPH)
		PSSR |= PSSR_OTGPH;

	UHCRHDA &= ~(RH_A_NPS);
	UHCRHDA |= RH_A_PSM;

	/* Set port power control mask bits, only 3 ports. */
	UHCRHDB |= (0x7<<17);
}

void usb_board_init_fail(void)
{
	return;
}

void usb_board_stop(void)
{
	UHCHR |= UHCHR_FHR;
	udelay(11);
	UHCHR &= ~UHCHR_FHR;

	UHCCOMS |= 1;
	udelay(10);

	CKEN &= ~CKEN10_USBHOST;

	puts("Called USB STOP\n");
	return;
}
