/*
 * cpu/pxa/cpu-freq-pxa27x.c
 *
 * cpufreq CPU driver for PXA27x, adopted from Linux' cpu-freq-bcd.c
 *
 *   31-Jul-2002 : Initial version [FB]
 *   29-Jan-2003 : added PXA255 support [FB]
 *   20-Aug-2003 : Added Bulverde suppprt[Cain Yuan]
 *   11-May-2005 : corrected some bugs when no DVM/DFM support [dv]
 *   24-May-2005 : imported to U Boot
 *
 * Copyright (C) 2002,2003 Intrinsyc Software
 * Copyright (C) 2003 Intel Coporation.
 * Copyright (c) 2005 Detlef Vollmann, Toradex AG, <dv@vollmann.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
#include <common.h>
#include <asm/arch/hardware.h>
#include <asm/arch/pxa-regs.h>

/* Since CPDIS and PPDIS is always the same, we use only one definition here.
*/
#define	PDIS	0	/* Core PLL and Peripheral PLL is enabled after FCS. */

/* Crystal clock: 13 MHZ */
#define BASE_CLK        13000000

static void pxa27x_setspeed(unsigned int CLKCFGValue);


/*
 *	Always remember N's value is always 2*N.
 */

void pxa27x_set_core_freq(int L, int N,
			  int turbo_mode_flag, int fast_bus_mode_flag,
			  int a)
{
	/* unsigned int mdrefr = 0; */
	int CLKCFGValue = 0;
	int turbo_mode = 0;
	int half = 0;

	/* mdrefr = MDREFR;		get the value. */
	/* First set CCCR (A = 1) */
	CCCR = (N << 7) | L | (PDIS << 30) | (PDIS << 31) | (a << 25);

	/* Check TURBO mode. */
	if (turbo_mode_flag != 0)	/* turbo_mode specified. */
		turbo_mode = 1;
	else if (N > 2)			/* Need turbo mode. */
		turbo_mode = 1;

	/* set up the new CLKCFG value, for an new frequency
	   0x2 is used to set FC bit in CLKCFG.	*/
	CLKCFGValue = 0x2 | (fast_bus_mode_flag << 3) | turbo_mode;

	half = 0;
	if (fast_bus_mode_flag) {	/* fast bus. */
		if (L >= 9)
			half = 1;
	} else {
		if (L >= 17)
			half = 1;
	}

	/*
	 * Program the CLKCFG (CP14, reigster 6) for frequency change
	 */
	pxa27x_setspeed(CLKCFGValue);
}

/*
 * This should be called with a valid freq point.
 */

static void pxa27x_setspeed(unsigned int CLKCFGValue)
{
	unsigned int unused;

	__asm__ __volatile__(" \n\
		ldr     r4, [%1]                        @load MDREFR \n\
		mcr     p14, 0, %2, c6, c0, 0           @ set CCLKCFG[FCS] \n\
		ldr     r5, =0xe3dfefff \n\
		and     r4, r4, r5      \n\
		str     r4,  [%1]                       @restore \n\
		"
		: "=&r" (unused)
		: "r" (&MDREFR), "r" (CLKCFGValue)
		: "r4", "r5");
}


int	pxa27x_get_freq(void)
{
	unsigned int freq, n, l, ccsr;

	if (CCCR & (1 << 31))
		return 13000;
	ccsr = CCSR;
	l = ccsr & 0x0000001F;		/* Get L */
	n = (ccsr & 0x00000780)>>7;	/* Get 2N */
	if (n < 2)
		n = 2;
	freq = (13 * l * n * 1000)>>1;	/* in kHz */
	return freq;
}

/*
 * Return the current lclk requency in units of 10kHz
 */
unsigned int get_lclk_frequency_10khz(void)
{
	return (CCSR & 0x1f) * BASE_CLK / 10000;
}
