/*
 *
 * Configuation settings for a Toradex Colibri XScale PXA270 module on a
 * Toradex Colibri Evaluation Board.
 *
 * (C) Copyright 2008
 * Marcel Ziswiler, Noser Engineering, marcel.ziswiler@noser.com.
 *
 * (C) Copyright 2005-2007
 * Detlef Vollmann, <dv@vollmann.ch>
 *
 * (C) Copyright 2004
 * Robert Whaley, Applied Data Systems, Inc. rwhaley@applieddata.net
 *
 * (C) Copyright 2002
 * Kyle Harris, Nexus Technologies, Inc. kharris@nexus-tech.net
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

/*
 * If we are developing, we might want to start U-Boot from RAM
 * so we MUST NOT initialize critical regs like mem-timing ...
 */
#ifdef DEBUG_RAMBOOT
#define CONFIG_SKIP_LOWLEVEL_INIT	1
#define CONFIG_SKIP_RELOCATE_UBOOT	1
#define CONFIG_SKIP_SDRAM_INIT		1
#endif

/*
 * High Level Configuration Options
 * (easy to change)
 */
#define CONFIG_PXA27X		1	/* This is an PXA27x CPU */
#define CONFIG_COLIBRI		1	/* on a Colibri module */
#define LITTLEENDIAN		1	/* used by usb_ohci.c */
#define CONFIG_MMC		1
#define BOARD_LATE_INIT		1

#undef CONFIG_USE_IRQ			/* we don't need IRQ/FIQ stuff */

#define CONFIG_TIMESTAMP
#define RTC

#define CONFIG_BOOTCOUNT_LIMIT		/* support for boot count limit */
				/* just after boot parameters
				   Note: internal SRAM not preserved! */
#define CFG_BOOTCOUNT_ADDR	0xA0000200
#undef CONFIG_ALTBOOT_GPIO		/* GPIO set to high activates
					   alternate boot command */

/*
 * Size of malloc() pool
 */
#define CFG_MALLOC_LEN		(CONFIG_ENV_SECT_SIZE + 128*1024)
/* size in bytes reserved for initial data */
#define CFG_GBL_DATA_SIZE	128

/*
 * Hardware drivers
 */

/*
 * select serial console configuration
 */
#define CONFIG_SERIAL_MULTI
#define CONFIG_FFUART		1	/* we use FFUART on Colibri */
#define CONFIG_BTUART		1	/* we use BTUART on Colibri */
#define CONFIG_STUART		1	/* we use STUART on Colibri */

/* allow to overwrite serial and ethaddr */
#define CONFIG_ENV_OVERWRITE

#define CONFIG_BAUDRATE		115200
				/* valid baudrates */
#define CFG_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200, 230400 }

#define CONFIG_DOS_PARTITION	1

/*
 * Command line configuration.
 */
#include <config_cmd_default.h>

#define CONFIG_CMD_CACHE
#define CONFIG_CMD_DHCP
#define CONFIG_CMD_DIAG
/*#define CONFIG_CMD_KGDB*/
#define CONFIG_CMD_FAT
#define CONFIG_CMD_IMLS
#define CONFIG_CMD_MMC
#define CONFIG_CMD_PING
#define CONFIG_CMD_SDRAM
#define CONFIG_CMD_USB

#undef CONFIG_SHOW_BOOT_PROGRESS

#define CONFIG_ETHADDR		00:00:00:00:00:00
#define CONFIG_NETMASK		255.255.255.0
#define CONFIG_IPADDR		192.168.1.2
#define CONFIG_SERVERIP		192.168.1.1

#define CONFIG_BOOTDELAY	5
#define CONFIG_SILENT_CONSOLE	1

/* set CONFIG_FASTBOOT_GPIO if you have a GPIO that is set to high if
   you want immediate boot without "press key" prompt.
 */
/* #define CONFIG_FASTBOOT_GPIO	37 */

#define CONFIG_BOOTCOMMAND	"bootm 80000 || dhcp && bootm"
#define CONFIG_BOOTARGS		"root=/dev/nfs ip=:::::eth0: console=ttyS0," \
				"115200n8"

#define CONFIG_SETUP_MEMORY_TAGS 1
#define CONFIG_CMDLINE_TAG	1	/* enable passing of ATAGs */
/* #define CONFIG_INITRD_TAG	1 */
#define CONFIG_SERIAL_TAG	1

#ifdef CONFIG_CMD_KGDB
#define CONFIG_KGDB_BAUDRATE	230400	/* speed to run kgdb serial port */
#define CONFIG_KGDB_SER_INDEX	2	/* which serial port to use */
#endif

/*
 * Miscellaneous configurable options
 */
#define CFG_HUSH_PARSER		1
#define CFG_PROMPT_HUSH_PS2	"> "

#define CFG_LONGHELP				/* undef to save memory */
#ifdef CFG_HUSH_PARSER
#define CFG_PROMPT		"u-boot$ "	/* Monitor Command Prompt */
#else
#define CFG_PROMPT		"u-boot=> "	/* Monitor Command Prompt */
#endif
#define CFG_CBSIZE		256		/* Console I/O Buffer Size */
#define CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define CFG_MAXARGS		16		/* max number of cmd args */
#define CFG_BARGSIZE		CFG_CBSIZE	/* Boot Argument Buffer Size */
#define CFG_DEVICE_NULLDEV	1

#define CFG_MEMTEST_START	0xa0400000	/* memtest works on */
#define CFG_MEMTEST_END		0xa0800000	/* 4 ... 8 MB in DRAM	*/

#undef CFG_CLKS_IN_HZ		/* everything, incl board info, in Hz */

#define CFG_LOAD_ADDR		0xa1000000	/* default load address */

#define CFG_HZ			3686400		/* incrementer freq 3.69 MHz */
/* need to look more closely, I think this is Turbo = 2x, L=91Mhz */
/* #define CFG_CPUSPEED		0x207 */
/* standard setting for 312MHz; L=16, N=1.5, A=0, SDCLK!=SystemBus */
#define CFG_CPUSPEED		0x190

#define CFG_MMC_BASE		0xF0000000

/*
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	(128*1024)	/* regular stack */
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ	(4*1024)	/* IRQ stack */
#define CONFIG_STACKSIZE_FIQ	(4*1024)	/* FIQ stack */
#endif /* CONFIG_USE_IRQ */

/*
 * Physical Memory Map
 */
/* we have 1 bank of DRAM, but we have room for 4 */
#define CONFIG_NR_DRAM_BANKS	4
#define PHYS_SDRAM_1		0xa0000000	/* SDRAM Bank #1 */
#define PHYS_SDRAM_1_SIZE	0x04000000	/* 64 MB */
#define PHYS_SDRAM_2		0xa4000000	/* SDRAM Bank #2 */
#define PHYS_SDRAM_2_SIZE	0x00000000	/* 0 MB */
#define PHYS_SDRAM_3		0xa8000000	/* SDRAM Bank #3 */
#define PHYS_SDRAM_3_SIZE	0x00000000	/* 0 MB */
#define PHYS_SDRAM_4		0xac000000	/* SDRAM Bank #4 */
#define PHYS_SDRAM_4_SIZE	0x00000000	/* 0 MB */

#define CFG_DRAM_BASE		0xa0000000
#define CFG_DRAM_SIZE		0x04000000

#define PHYS_FLASH_1		0x00000000	/* Flash Bank #1 */
#define PHYS_FLASH_SIZE		0x02000000	/* 32 MB */
#define PHYS_FLASH_BANK_SIZE	0x02000000	/* 32 MB Banks */
#define PHYS_FLASH_SECT_SIZE	0x00040000	/* 256 KB sectors (x2) */

#define CFG_FLASH_BASE		PHYS_FLASH_1
#define CFG_MONITOR_BASE	CFG_FLASH_BASE
#define CFG_FLASH_CFI		1
#define CONFIG_FLASH_CFI_DRIVER	1
#define CFG_FLASH_CFI_WIDTH	FLASH_CFI_32BIT
#define CFG_FLASH_USE_BUFFER_WRITE 1 /* write flash less slowly */
#define CFG_FLASH_PROTECTION	1

/* #define CFG_DEEP_DEBUG	1 */

/*
 * Davicom DM9000 Network Card
 */
#define CONFIG_DRIVER_DM9000	1
#define CONFIG_DM9000_BASE	0x08000000	/* CS2 */
#define DM9000_IO		CONFIG_DM9000_BASE
#define DM9000_DATA		(CONFIG_DM9000_BASE + 4)
#define CONFIG_DM9000_USE_32BIT	1
#define CONFIG_ETH_ADDR_IN_ENV	1
#undef  CONFIG_SHOW_ACTIVITY
#define CONFIG_NET_RETRY_COUNT	10		/* # of retries */

#define CONFIG_USB_OHCI_NEW	1
#define CFG_USB_OHCI_BOARD_INIT	1
#define CFG_USB_OHCI_MAX_ROOT_PORTS	3
#define CFG_USB_OHCI_REGS_BASE	0x4C000000
#define CFG_USB_OHCI_SLOT_NAME	"colibri"
#define CONFIG_USB_STORAGE	1
#define CFG_USB_OHCI_CPU_INIT	1

/*
 * GPIO settings
 */
#if 1 /* minimal colibri settings, includes FFUART and Ethernet */
#define CFG_GPSR3_VAL	0x00000000
#define CFG_GPSR2_VAL	0x0000c000
#define CFG_GPSR1_VAL	0x00020000
#define CFG_GPSR0_VAL	0x00000000

#define CFG_GPCR3_VAL	0x0
#define CFG_GPCR2_VAL	0x0
#define CFG_GPCR1_VAL	0x0
#define CFG_GPCR0_VAL	0x0

#define CFG_GPDR3_VAL	0x01000000	/* USB: Set GPIO120 to out */
#define CFG_GPDR2_VAL	0x0200c000	/* USB: Set GPIO89 to out */
#define CFG_GPDR1_VAL	0x00020181  /* MMC: Set GPIO32 to out*/
#define CFG_GPDR0_VAL	0x08000000

#define CFG_GAFR3_U_VAL	0x00024001	/* USB: Set GPIO119 to a.f. 1 and GPIO120 to a.f. 2 */ /* MMC: Set GPIO112 to a.f. 1 */
#define CFG_GAFR3_L_VAL	0x54000200  /* MMC: Set GPIO109 to a.f. 1 and GPIO110 to a.f. 1 and GPIO111 to a.f. 1 */
#define CFG_GAFR2_U_VAL	0x01090000	/* USB: Set GPIO88 to a.f. 1 and GPIO89 to a.f. 2 */ /* MMC: Set GPIO92 to a.f. 1 */
#define CFG_GAFR2_L_VAL	0xa0000000
#define CFG_GAFR1_U_VAL	0x00000008
#define CFG_GAFR1_L_VAL	0x0002901A  /* MMC: Set GPIO32 to a.f. 2 */
#define CFG_GAFR0_U_VAL	0x00c00010
#define CFG_GAFR0_L_VAL	0x00100000

#else /* maximal colibri settings for Colibri eval board */
#define CFG_GPSR3_VAL	0x00000002
#define CFG_GPSR2_VAL	0x8000c000
#define CFG_GPSR1_VAL	0x00cf0000
#define CFG_GPSR0_VAL	0x00000000

#define CFG_GPCR3_VAL	0x0
#define CFG_GPCR2_VAL	0x0
#define CFG_GPCR1_VAL	0x0
#define CFG_GPCR0_VAL	0x0

#define CFG_GPDR3_VAL	0x00000004
#define CFG_GPDR2_VAL	0x8020ffff
#define CFG_GPDR1_VAL	0xfccf2981
#define CFG_GPDR0_VAL	0xc8000000

#define CFG_GAFR3_U_VAL	0x00000001
#define CFG_GAFR3_L_VAL	0x10010210
#define CFG_GAFR2_U_VAL	0x41000400
#define CFG_GAFR2_L_VAL	0xaaaaaaaa
#define CFG_GAFR1_U_VAL	0xaaa5a0aa
#define CFG_GAFR1_L_VAL	0x0992901a
#define CFG_GAFR0_U_VAL	0xa5c00010
#define CFG_GAFR0_L_VAL	0x00100000
#endif

#define CFG_PSSR_VAL		0x30

/*
 * Clock settings
 */
#define CFG_CKEN		0x00400240	/* what is the 40? */
#define CFG_CCCR		0x02000190	/* 312Mhz */
/* #define CFG_CCCR		0x02000290	520Mhz */
/* #define CFG_CCCR		0x02000210	416 Mhz */

/*
 * Memory settings
 */
/* set CS1 static memory control register for external chip selects (e.g. CAN):
   RBUFFx 0b0 = return data buffer
   RRRx 0b111 = ROM/SRAM recovery time of 7
   RDNx 0b1111 = 30 clocks ROM delay next access
   RDFx 0b1111 = 30 clocks ROM delay first access
   RBW 0b0 = ROM bus width of 32 bits
   RTx 0b100 = variable-latency I/O (VLIO)
   set CS0 static memory control register for NOR flash:
   RBUFFx 0b1 = streaming behavior
   RRRx 0b100 = ROM/SRAM recovery time of 4
   RDNx 0b0101 = 5 clocks ROM delay next access
   RDFx 0b1111 = 30 clocks ROM delay first access
   RBW 0b0 = ROM bus width of 32 bits
   RTx 0b010 = burst of 4 ROM or flash (with non-burst writes) */
#define CFG_MSC0_VAL		0x7ff4c5f2
/* set CS3 static memory control register for external Ethernet chip (Protea):
   RBUFFx 0b1 = streaming behavior
   RRRx 0b000 = ROM/SRAM recovery time of 0
   RDNx 0b1001 = 9 clocks ROM delay next access
   RDFx 0b1010 = 10 clocks ROM delay first access
   RBW 0b1 = ROM bus width of 16 bits
   RTx 0b001 = SRAM
   set CS2 static memory control register for Ethernet chip:
   RBUFFx 0b1 = streaming behavior
   RRRx 0b111 = ROM/SRAM recovery time of 7
   RDNx 0b1001 = 9 clocks ROM delay next access
   RDFx 0b0111 = 7 clocks ROM delay first access
   RBW 0b0 = ROM bus width of 32 bits
   RTx 0b100 = variable-latency I/O (VLIO) */
#define CFG_MSC1_VAL		0x89a9f974
#define CFG_MSC2_VAL		0x00000000
#define CFG_MDCNFG_VAL		0x08000AC8
#define CFG_MDREFR_VAL		0x0000001E
#define CFG_MDMRS_VAL		0x00000000

#define CFG_FLYCNFG_VAL		0x00010001
#define CFG_SXCNFG_VAL		0x40044004
				/* factory default for PXA is 5 */
#define CFG_MEM_BUF_IMP		0x0f

/*
 * PCMCIA and CF Interfaces
 */
#define CFG_MECR_VAL		0x00000001
#define CFG_MCMEM0_VAL		0x00014307
#define CFG_MCMEM1_VAL		0x00014307
#define CFG_MCATT0_VAL		0x0001C787
#define CFG_MCATT1_VAL		0x0001C787
#define CFG_MCIO0_VAL		0x0001430F
#define CFG_MCIO1_VAL		0x0001430F

/*
 * FLASH and environment organization
 */
#define CFG_MAX_FLASH_BANKS	1	/* max number of memory banks */
#define CFG_MAX_FLASH_SECT	136	/* max number of sectors on one chip */

/* timeout values are in ticks */
#define CFG_FLASH_ERASE_TOUT	(100*CFG_HZ) /* Timeout for Flash Erase */
#define CFG_FLASH_WRITE_TOUT	(100*CFG_HZ) /* Timeout for Flash Write */

/* Flash environment locations */
#define CONFIG_ENV_IS_IN_FLASH	1
#define CONFIG_FIX_PARAM_SIZE	256
				/* Addr of Environment Sector */
#define CONFIG_ENV_ADDR		(PHYS_FLASH_1 + PHYS_FLASH_SECT_SIZE + CONFIG_FIX_PARAM_SIZE)
				/* Total Size of Environment */
#define CONFIG_ENV_SIZE		(PHYS_FLASH_SECT_SIZE - CONFIG_FIX_PARAM_SIZE - CONFIG_FIX_PARAM_SIZE)
				/* Total Size of Environment Sector */
#define CONFIG_ENV_SECT_SIZE	PHYS_FLASH_SECT_SIZE

#endif	/* __CONFIG_H */
